"""
IDaaS Python SDK - Factory Module

This module contains factory classes for creating credential providers.
"""

from cloud_idaas.core.factory.idaas_credential_provider_factory import IDaaSCredentialProviderFactory

__all__ = [
    "IDaaSCredentialProviderFactory",
]
